#pragma once

#include <QMainWindow>
#include <QDir>
#include <QStringLiteral>

class AbstractViewer;
class RecentFiles;
class ViewerFactory;

namespace Ui {
class MainWindow;
}

class ScribbleArea;
class GraphWidget;

class MainWindow : public QMainWindow
{
    Q_OBJECT

public:
    explicit MainWindow(QWidget *parent = nullptr);
    ~MainWindow();

public slots:
    bool openFile(const QString &fileName);

private slots:
    void onActionOpenTriggered();
    void onActionAboutTriggered();
    void onActionAboutQtTriggered();

private:
    void readSettings();
    void saveSettings() const;
    void restoreViewerSettings();
    void resetViewer() const;
    void saveViewerSettings() const;

    std::unique_ptr<ScribbleArea> m_scribble_area;
    std::unique_ptr<GraphWidget> m_graph_widget;

    QDir m_currentDir;
    AbstractViewer *m_viewer = nullptr;
    //std::unique_ptr<Ui::MainWindow> ui;
    //std::unique_ptr<RecentFiles> m_recentFiles;
    //std::unique_ptr<ViewerFactory> m_factory;

    static constexpr QLatin1StringView settingsDir = QLatin1StringView("WorkingDir");
    static constexpr QLatin1StringView settingsMainWindow = QLatin1StringView("MainWindow");
    static constexpr QLatin1StringView settingsViewers = QLatin1StringView("Viewers");
    static constexpr QLatin1StringView settingsFiles = QLatin1StringView("RecentFiles");
};